#!/usr/local/bin/python2.7

def login(username, password):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST
		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sql = "SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/" + username + "> WHERE { ?subject ?predicate ?object } limit 1"
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sparql.setQuery(sql)
		results = sparql.query().convert()
		return True
	except Exception, e:
		return False
	return True

settings = {}
settings['carreSensorsOntologyURL'] = "http://carre.kmi.open.ac.uk/ontology/sensors.owl" #not used directly

measurements = {}
measurements['Calories'] = settings['carreSensorsOntologyURL'] + '#has_calories'
measurements['Date'] = settings['carreSensorsOntologyURL'] + '#has_date'
measurements['CaloriesBMR'] = settings['carreSensorsOntologyURL'] + '#has_calories_b_m_r'
measurements['ActivityCalories'] = settings['carreSensorsOntologyURL'] + '#has_activity_calories'
measurements['MarginalCalories'] = settings['carreSensorsOntologyURL'] + '#has_marginal_calories'
measurements['SedentaryActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_sedentary_activity_duration'
measurements['LightActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_light_activity_duration'
measurements['StartDate'] = settings['carreSensorsOntologyURL'] + '#has_start_date'
measurements['SleepStatus'] = settings['carreSensorsOntologyURL'] + '#has_sleep_status'
measurements['Steps'] = settings['carreSensorsOntologyURL'] + '#has_steps'
measurements['Distance'] = settings['carreSensorsOntologyURL'] + '#has_distance'
measurements['Elevation'] = settings['carreSensorsOntologyURL'] + '#has_elevation'
measurements['Floors'] = settings['carreSensorsOntologyURL'] + '#has_floors'
measurements['ModerateActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_moderate_activity_duration'
measurements['LightActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_light_activity_distance'
measurements['ModerateActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_moderate_activity_distance'
measurements['TrackedActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_tracked_activity_distance'
measurements['IntenseActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_intense_activity_duration'
measurements['IntenseActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_intense_activity_distance'
measurements['TimesAwake'] = settings['carreSensorsOntologyURL'] + '#has_times_awake'
measurements['TimesRestless'] = settings['carreSensorsOntologyURL'] + '#has_times_restless'
measurements['TimeToFallAsleep'] = settings['carreSensorsOntologyURL'] + '#has_time_to_fall_asleep'
measurements['SleepTime'] = settings['carreSensorsOntologyURL'] + '#has_sleep_time'
measurements['SleepEfficiency'] = settings['carreSensorsOntologyURL'] + '#has_sleep_efficiency'
measurements['AwakeDuration'] = settings['carreSensorsOntologyURL'] + '#has_awake_duration'
measurements['AsleepDuration'] = settings['carreSensorsOntologyURL'] + '#has_asleep_duration'
measurements['RestlessDuration'] = settings['carreSensorsOntologyURL'] + '#has_restless_duration'
measurements['MinutesAfterWakeup'] = settings['carreSensorsOntologyURL'] + '#has_minutes_after_wakeup'


def __getQuery__(predicate, startDate ,username):
	sql = """
		SELECT max(?v1) as ?value xsd:date(?d) as ?d FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
		{
		         ?subject ?predicate ?object.
		         ?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date> ?d1.
		         ?d1 <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?d.
                         ?subject <"""+predicate+"""> ?v.
                         ?v <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?v1.
		         FILTER ( ( xsd:date(?d) >= '""" + startDate + """'^^xsd:date) )
		}
		GROUP BY xsd:date(?d)
	"""
	return sql


def getStartDate(period=365):
	import pandas as pd
	from datetime import date, timedelta
	d = pd.datetime.today() - timedelta(days=period)
	d = str(d.year) + d.strftime('%m') + d.strftime('%d')
	return d

def getPredicate(measurement):
	global measurements
	predicate = None
	if measurement in measurements.keys():
			predicate = measurements[measurement]
	elif measurement in measurements.values():
		predicate = measurement

	return predicate


def getDataContinuous(measurement, username=None, password=None,token=None, period=365):
	import pandas as pd
	import json
	global measurements
	try:
		predicate = getPredicate(measurement)
		if predicate is None:
			return predicate

		rs = {}
		d = getStartDate(period)
		if username is None:
			import virtuoso
			username = virtuoso.getUsernameFromToken(token)
		sql = __getQuery__(predicate, d, username)
		if password is None:
			import virtuoso
			rs = virtuoso.executeSPARQL(sql, token)
		else:
			rs = sparql(sql, username, password)
		items = json.loads(rs)
		df = pd.DataFrame()
		for item in items:
			d = {}
			d[item['d']['value']] = item['value']['value']
			tmp = pd.DataFrame.from_dict(d.items())
			df = pd.concat([df, tmp])
		df['date'] = df[0].apply(pd.to_datetime)
		df.rename(columns={1: 'value'}, inplace=True)
		df = df[['date', 'value']]

		df['value'] = df['value'].astype(float)

		df = df.sort('date')
		
		df = df.reset_index().sort('date')
		del df['index']
	except Exception, e:
		return str(e)
	return df

def getAveragesDataContinuous(measurement, username, password, period=365, grouping="M"):
	df = getDataContinuous(measurement, username=username, password=password, period=period)
	df = df.set_index('date')
	davg = df.resample('M', how='mean')
	return df

def fetchCaloriesPie(username, password):
	import gviz_api
	calories = getDataContinuous("Calories", username = username,password = password)
	caloriesBMR = getDataContinuous("CaloriesBMR", username = username,password = password)
	activityCalories = getDataContinuous("ActivityCalories", username = username,password = password)

	description = {"calories": ("number", "Calories"),
			   "caloriesBMR": ("number", "CaloriesBMR"),
			   "activityCalories": ("number", "ActivityCalories")}
	data = [{"Calories": 3000, "caloriesBMR": 2000, "ActivityCalories": 1000},
			{"Calories": 5000, "caloriesBMR": 3000, "ActivityCalories": 1000}]
	data_table = gviz_api.DataTable(description)
	data_table.LoadData(data)
	json_description = data_table.ToJSCode("json_description",
							   columns_order=("calories", "caloriesBMR", "activityCalories"),
							   order_by="calories")
	json = data_table.ToJSon(columns_order=("calories", "caloriesBMR", "activityCalories"),
						   order_by="calories")
	return (json_description, json)


def sparql(q, username, password, format='JSON'):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
		import json

		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)

		sparql.setQuery(q)
		if format.lower()=='json':
			sparql.setReturnFormat(JSON)
		elif format.lower()=='turtle':
			sparql.setReturnFormat(TURTLE)
		elif format.lower()=='xml':
			sparql.setReturnFormat(XML)
		elif format.lower()=='n3':
			sparql.setReturnFormat(N3)
		else:
			return "invalid format. Select one of the following: JSON (default), XML, TURTLE, N3, RDF"
		results = sparql.query().convert()
		
	except Exception, e:
		return str(e)
	if format.lower()=='json':
		return json.dumps(results['results']['bindings'])
	elif format.lower()=='turtle':
		return results
	elif format.lower()=='xml':
		return results.toxml()
	elif format.lower()=='n3':
		return results


def __sql_available_measurements__(username):
	sql = """SELECT distinct ?predicate FROM <https://carre.kmi.open.ac.uk/users/"""+username+"""> WHERE 
			{ ?subject ?predicate ?object .
			FILTER (REGEX(STR(?predicate), "http://carre.kmi.open.ac.uk/ontology/sensors.owl", "i")) 
			}
			"""
	return sql


def __listAvailableMeasurements__(username=None, password=None, token=None):
	import json
	import virtuoso
	if username is None:
		username = virtuoso.getUsernameFromToken(token)
	sql = __sql_available_measurements__(username)
	if password is None:
		st = virtuoso.executeSPARQL(sql, token)
		l = json.loads(st)
	else:
		st = sparql(sql, username, password)
		l = json.loads(st)
	predicates = []
	for item in l:
		predicates.append(item['predicate']['value'])
	if 'http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date' in predicates:
		predicates.remove('http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date')

	return predicates

def getAllMeasurements(username=None, password=None, token=None):
	import pandas as pd
	predicates = __listAvailableMeasurements__(username=username, password=password, token=token)
	df = pd.DataFrame()
	for predicate in predicates:
		tmp = getDataContinuous(predicate, username = username, password = password, token = token)
		if tmp is None:
			continue

		if len(tmp)==0:
			continue

		if isinstance(tmp, basestring):
			continue

		tmp.rename(columns={'value': predicate}, inplace=True)
		
		if len(df)==0:
			df = tmp
		else:
			df = pd.merge(tmp, df, left_on='date', right_on='date', how='outer')

	return df

def createICal(username=None, password=None, token=None):
	from icalendar import Calendar, Event
	import uuid
	import numpy as np

	df = getAllMeasurements(username, password, token)
	if 'date' in df.columns:
		df['date'] = df['date'].apply(lambda x: x.strftime('%Y%m%d'))
		df = df.set_index('date')
	
	global measurements
	im = {v: k for k, v in measurements.items()}
	columns = df.columns.tolist()
	

	cal = Calendar()
	cal.add('version', '2.0')
	cal.add('PRODID','George Gkotsis george.gkotsis@open.ac.uk//KMi, OU, UK//EN')
	
	for row in df.iterrows():
		event = Event()
		event['uid'] = str(uuid.uuid4())
		event['dtstart'] = row[0]
		event['dtend'] = row[0]
		summary = []
		description = []
		for measurement in columns:
			v = row[1][measurement]
			if np.isnan(v):
				continue
			summary.append(im[measurement] + " " + str(v))
			if im[measurement]== "SleepTime":
				m, s = divmod(v, 60)
				h, m = divmod(m, 60)
				if m!=0:
					description.append("Sleep Time: " + str(int(h)) + " hours, " + str(int(m))+ " minutes")
				else:
					description.append("Sleep Time: " + str(int(h)) + " hours")
			elif im[measurement]== "ActivityCalories":
				description.append("Activity: " + str(int(v))+ " Calories")
			elif im[measurement]== "Distance":
				description.append(im[measurement] + ": " + str(round(v,2)) + " kms")	
			elif im[measurement]== "Steps":
				description.append(im[measurement] + ": " + str(int(v)))
			else:
				description.append(im[measurement] + ": " + str(v))
		summary = " ".join(summary)
		description.sort()
		description = " \n".join(description)
		event['summary'] = 'Activity Digest'
		event['description'] = description
		cal.add_component(event)


	return cal.to_ical()

def createHTML(url, body):
	import BeautifulSoup
	d = BeautifulSoup.BeautifulSoup()
	d.append(body)
	body = d.prettify()
	body = body.replace("\n", "<br/>")


	head = """
	<!DOCTYPE html>
	<html>
  		<head>
    	<title>""" + url +"""</title>
    	<meta charset="utf-8">
    	<meta name="viewport" content="width=device-width, initial-scale=1">
    	<link rel="stylesheet" href="http://maxcdn.bootstrapcdn.com/bootstrap/3.3.1/css/bootstrap.min.css">
  		</head>

	<body>
	<br/><br/><br/>
    <div class="container">
        <div class="col-md-12 col-lg-12 row">
	<p class="lead">
	"""

	foot = """
		</p>
		</div>
	</div>
	    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
    	<script src="http://maxcdn.bootstrapcdn.com/bootstrap/3.3.1/js/bootstrap.min.js"></script>
  	</body>
	</html>
	"""
	html = head + body + foot




	return html

def describe(url, username, password):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
		import json

		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sql = "DESCRIBE <" + url + "> FROM <https://carre.kmi.open.ac.uk/users/" + username + ">"
		sparql.setQuery(sql)
		sparql.setReturnFormat(N3)
		results = sparql.query().convert()

		html = createHTML(url, results)
	except Exception, e:
		return "unknown error"

	return html
