# oauth.Helper

import tripleStore.oauth.settings as osettings

def generate_timestamp():
	import time
	"""Get seconds since epoch (UTC).
	Per `section 3.2.1`_ of the MAC Access Authentication spec.
	.. _`section 3.2.1`: http://tools.ietf.org/html/draft-ietf-oauth-v2-http-mac-01#section-3.2.1
	"""
	return unicode(int(time.time())*1000)

def generate_nonce():
	import random
	"""Generate parseudorandom nonce that is unlikely to repeat.
	Per `section 3.2.1`_ of the MAC Access Authentication spec.
	A random 64-bit number is appended to the epoch timestamp for both
	randomness and to decrease the likelihood of collisions.
	.. _`section 3.2.1`: http://tools.ietf.org/html/draft-ietf-oauth-v2-http-mac-01#section-3.2.1
	"""
	return unicode(unicode(random.getrandbits(74)) + generate_timestamp())


def getVirtuosoAppTokens(username):
	import tripleStore.VirtuosoHelper as VH
	uid = VH.getUserByUsername(username)[0]
	sql = "select a_key,a_secret from OAUTH..APP_REG where a_owner=" + str(uid)
	rs = VH._fetchTriples(sql, "INSERT_ROOT_USERNAME", "INSERT_ROOT_PASSWORD")
	key = rs['a_key'].tolist()[0]
	secret = rs['a_secret'].tolist()[0]
	return key, secret

def make_authorization_url(provider, username = None):
	# Generate a random string for the state parameter
	# Save it for use later to prevent xsrf attacks
	from uuid import uuid4
	import urllib
	if provider=="ihealth":
		CLIENT_ID = osettings.settings[provider + '-clientid']
		REDIRECT_URI = osettings.settings[provider + '-callback']
		AUTHORIZATION_URI = osettings.settings[provider + '-authorizationurl1']

		state = str(uuid4())
		params = {"client_id": CLIENT_ID,
			"response_type": "code",
			"state": state,
			"redirect_uri": REDIRECT_URI,
			"duration": "temporary",
			"scope": "identity"}
		url = AUTHORIZATION_URI + urllib.urlencode(params)
		return url
	elif provider=="virtuoso":
		import requests
		from requests_oauthlib import OAuth1
		import oauthlib
		
		request_token_url = osettings.settings[provider + "-authorizationurl1"]# "http://carre.kmi.open.ac.uk:8890/OAuth/request_token"
		client_key, client_secret = getVirtuosoAppTokens(username) #user-specific


		client = oauthlib.oauth1.Client(client_key=client_key, client_secret=client_secret, signature_type=oauthlib.oauth1.SIGNATURE_TYPE_QUERY)
		uri, headers, body = client.sign(request_token_url)


		c = requests.get(uri).content

		from urlparse import parse_qs
		d = parse_qs(c)
		d['oauth_token'] = d['oauth_token'][0]
		d['oauth_token_secret'] = d['oauth_token_secret'][0]
		return d
	elif provider=="fitbit":
		from fitbit.api import FitbitOauthClient
		CLIENT_ID = osettings.settings[provider + '-clientid']
		REDIRECT_URI = osettings.settings['carreSecureURL']
		SIGNATURE = osettings.settings[provider + '-secret']
		client = FitbitOauthClient(CLIENT_ID, SIGNATURE, callback_uri = osettings.settings['carreSecureURL']+"/devices/oauth/fitbit/callback")
		token = client.fetch_request_token()
		client.oauth.callback_uri = osettings.settings['carreSecureURL']+"/devices/oauth/fitbit/callback"
		
		url = client.authorize_token_url()
		url = url + "&oauth_callback="+osettings.settings['carreSecureURL']+"/devices/oauth/fitbit/callback"
		return url, token
	elif provider=="withings":
		import requests
		from requests_oauthlib import OAuth1
		import oauthlib
		client_key = osettings.settings[provider + '-clientid']
		client_secret = osettings.settings[provider + '-secret']
		request_token_url = osettings.settings[provider + "-authorizationurl1"]
		authorize_url = osettings.settings[provider + "-authorizationurl2"]
		cb = osettings.settings['carreSecureURL']+"/devices/oauth/withings/callback"
		client = oauthlib.oauth1.Client(client_key=client_key,client_secret=client_secret,callback_uri=cb, signature_type=oauthlib.oauth1.SIGNATURE_TYPE_QUERY)
		uri, headers, body = client.sign(request_token_url)
		response = requests.get(uri)
		from urlparse import parse_qs
		token = parse_qs(response.content)
		token['oauth_token'] = token['oauth_token'][0]
		token['oauth_token_secret'] = token['oauth_token_secret'][0]

		client = oauthlib.oauth1.Client(client_key=client_key, resource_owner_key=token['oauth_token'],resource_owner_secret=token['oauth_token_secret'],client_secret=client_secret, signature_type=oauthlib.oauth1.SIGNATURE_TYPE_QUERY)
		uri, headers, body = client.sign(authorize_url)
		return uri, token


		from withings import WithingsAuth, WithingsApi
		CLIENT_ID = osettings.settings[provider + '-clientid']
		SIGNATURE = osettings.settings[provider + '-secret']
		auth = WithingsAuth(CLIENT_ID, SIGNATURE)
		authorize_url = auth.get_authorize_url() + "&oauth_callback=" + cb
		token = {}
		token['oauth_token'] = auth.oauth_token
		token['oauth_token_secret'] = auth.oauth_secret


		return authorize_url, token
	elif provider=="medisana":
		import tripleStore.oauth.vitadock as vitadock
		app_key = osettings.settings[provider + "-clientid"]
		app_secret = osettings.settings[provider + "-secret"]

		c = vitadock.VitadockOauthClient(app_key, app_secret)
		c.getRequestTokens()
		return c.request_token, c.request_token_secret
	elif provider=="misfit":
		CLIENT_ID = osettings.settings[provider + '-clientid']
		url = osettings.settings['misfit-authorizationurl1'] + "&client_id=" + CLIENT_ID
		return url
	elif provider=="gcalendar":
		from oauth2client.client import flow_from_clientsecrets
		jsonFile = osettings.settings['google' + '-jsonpath']
		flow = flow_from_clientsecrets(jsonFile, scope='https://www.googleapis.com/auth/calendar',redirect_uri=osettings.settings['carreSecureURL'] + "/devices/google/oauth2callback")
		flow.params['access_type'] = 'offline'
  		flow.params['approval_prompt'] = 'force'
		url = flow.step1_get_authorize_url()
		return url
	elif provider=="googlefit":
		from oauth2client.client import flow_from_clientsecrets
		jsonFile = osettings.settings['googlefit' + '-jsonpath']
		flow = flow_from_clientsecrets(jsonFile, scope='https://www.googleapis.com/auth/fitness.activity.read https://www.googleapis.com/auth/fitness.body.read https://www.googleapis.com/auth/fitness.location.read',redirect_uri=osettings.settings['carreSecureURL'] + "/devices/oauth/googlefit/callback")
		flow.params['access_type'] = 'offline'
  		flow.params['approval_prompt'] = 'force'		
		url = flow.step1_get_authorize_url()
		return url
		
		

def get_token(username, provider, verifier, oauth_token=None, oauth_token_secret=None):
	CLIENT_ID = osettings.settings[provider + '-clientid']
	CLIENT_SECRET = osettings.settings[provider + '-secret']
	if provider=="ihealth":
		import requests
		REDIRECT_URI = osettings.settings[provider + '-landing']
		AUTHORIZATION_URI = osettings.settings[provider + '-authorizationurl2']
		client_auth = requests.auth.HTTPBasicAuth(CLIENT_ID, CLIENT_SECRET)
		post_data = {"grant_type": "authorization_code",
				 "code": verifier,
				 "client_id": CLIENT_ID,
				 "client_secret": CLIENT_SECRET,
				 "redirect_uri": REDIRECT_URI}
		response = requests.get(AUTHORIZATION_URI,params=post_data)
		token_json = response.json()
		return token_json
	elif provider=="withings":
		from withings import WithingsAuth, WithingsApi
		auth = WithingsAuth(CLIENT_ID, CLIENT_SECRET)
		auth.oauth_secret = oauth_token_secret
		auth.oauth_token = oauth_token
		creds = auth.get_credentials(verifier)
		rs = {}
		rs['access_token'] = creds.access_token
		rs['access_token_secret'] = creds.access_token_secret
		rs['user_id'] = creds.user_id

		#add notification
		client = WithingsApi(creds)
		client.subscribe(callback_url=osettings.settings['carreSecureURL'] + "/tomcat/responder/withings", comment="CARRE devices", appli = 0)
		client.subscribe(callback_url=osettings.settings['carreSecureURL'] + "/tomcat/responder/withings", comment="CARRE devices", appli = 1)
		client.subscribe(callback_url=osettings.settings['carreSecureURL'] + "/tomcat/responder/withings", comment="CARRE devices", appli = 4)
		client.subscribe(callback_url=osettings.settings['carreSecureURL'] + "/tomcat/responder/withings", comment="CARRE devices", appli = 16)
		client.subscribe(callback_url=osettings.settings['carreSecureURL'] + "/tomcat/responder/withings", comment="CARRE devices", appli = 32)
		return rs
	elif provider=="fitbit":
		from fitbit.api import FitbitOauthClient
		import fitbit
		client = FitbitOauthClient(CLIENT_ID, CLIENT_SECRET)
		client.resource_owner_key = oauth_token
		client.resource_owner_secret = oauth_token_secret
		token = client.fetch_access_token(verifier)
		
		#add listener
		f = fitbit.Fitbit(CLIENT_ID, CLIENT_SECRET)
		client.resource_owner_key = token['oauth_token']#oauth_token
		client.resource_owner_secret = token['oauth_token_secret']#oauth_token_secret
		f.client = client
		subscription_id = username
		subscriber_id = "1"
		f.subscription(subscription_id, subscriber_id)

		return token
	elif provider=="misfit":
		import requests
		import json
		url = osettings.settings[provider + "-authorizationurl2"]
		data = {}
		data['grant_type'] = "authorization_code"
		data['code'] = verifier
		data['redirect_uri'] = "https://carre.kmi.open.ac.uk/devices/oauth/misfit/callback"
		data['client_id'] = osettings.settings[provider + '-clientid']
		data['client_secret'] = osettings.settings[provider + '-secret']
		# print data
		response = requests.post(url, data=data)
		d = json.loads(response.content)
		token = d['access_token']
		r = requests.get("https://api.misfitwearables.com/move/resource/v1/user/me/profile", headers={"access_token":token})
		user = json.loads(r.content)
		return token, user
	elif provider=="google":
		from oauth2client.client import flow_from_clientsecrets
		jsonFile = osettings.settings['google' + '-jsonpath']
		flow = flow_from_clientsecrets(jsonFile, scope='https://www.googleapis.com/auth/calendar',redirect_uri=osettings.settings['carreSecureURL'] + "/devices/google/oauth2callback")
		credentials = flow.step2_exchange(verifier)
		import simplejson
		parsed_data = simplejson.loads(credentials.to_json())
		token = parsed_data['access_token']
		return token
	elif provider=="googlefit":
		from oauth2client.client import flow_from_clientsecrets
		jsonFile = osettings.settings[provider + '-jsonpath']
		flow = flow_from_clientsecrets(jsonFile, scope='https://www.googleapis.com/auth/fitness.activity.read https://www.googleapis.com/auth/fitness.body.read https://www.googleapis.com/auth/fitness.location.read',redirect_uri=osettings.settings['carreSecureURL'] + "/devices/oauth/googlefit/callback")
		flow.params['access_type'] = 'offline'
  		flow.params['approval_prompt'] = 'force'
		credentials = flow.step2_exchange(verifier)
		import simplejson
		# approval_prompt=force
		parsed_data = simplejson.loads(credentials.to_json())
		token = parsed_data['access_token']
		token_expiry = parsed_data['token_expiry']
		refresh_token = parsed_data['refresh_token']
		expires_in = parsed_data['token_response']['expires_in']
		return token, token_expiry, expires_in, refresh_token
	elif provider=="medisana":
		import tripleStore.oauth.vitadock as vitadock
		app_key = osettings.settings[provider + "-clientid"]
		app_secret = osettings.settings[provider + "-secret"]		
		c = vitadock.VitadockOauthClient(app_key, app_secret)
		c.request_token = oauth_token
		c.request_token_secret = oauth_token_secret
		c.getAccessTokens(verifier)
		return c.access_token, c.access_token_secret
		

def storeWithingsMeasurements(username, password):
	from withings import WithingsAuth, WithingsApi, WithingsCredentials
	import tripleStore.VirtuosoHelper as VH
	CLIENT_ID = osettings.settings['withings-clientid']
	SECRET = osettings.settings['withings-secret']
	graphName = "<" + osettings.settings['carreSecureURL'] + "/users/" + username  + ">"

	sql = "SPARQL SELECT ?connection ?predicate ?object FROM " + graphName + " WHERE { ?connection <" + osettings.settings['carreManufacturerPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/withings> . ?connection ?predicate ?object}"
	data = VH._fetchTriples(sql, username, password)
	if data is None:
		return -1
	uid = int(data[data.predicate==osettings.settings['carreUserIdPredicate']].object.tolist()[0])
	atoken = data[data.predicate==osettings.settings['carreAccessTokenPredicate']].object.tolist()[0]
	atokensecret = data[data.predicate==osettings.settings['carreAccessTokenSecretPredicate']].object.tolist()[0]
	creds = WithingsCredentials(access_token=atoken, access_token_secret=atokensecret, consumer_key=CLIENT_ID, consumer_secret=SECRET, user_id=uid)
	client = WithingsApi(creds)
	measures = client.get_measures(limit=1)
	#todo: measures is a type list
	return measures

